within FlueGasTreatment.Reactions.SCM.Templates;
partial model PartialShrinkingCoreModel "SCM with spheric particles"
   extends FlueGasTreatment.Reactions.SCM.Templates.PartialSCM;

  import Modelon.Math.Smoothing.regExp;
  import ThermalPower.FlueGasTreatment.Internal.Conversion.to_J;
  import Modelica.Constants.R;
  import Modelica.Constants.eps;
  import ThermalPower.FlueGasTreatment.Internal.Conversion.from_Nm3;

  input Modelica.Units.SI.Concentration c_g_in=600*1e-6/from_Nm3(1,p,T)/Gas.MMC[3]
                                                    "SO2 concerntration in the ambient gas" annotation(dialog(Enable=true));

  input Real RH(min= 0, max= 1) = 0.5 "Relative humidity (molar fraction)";
  input Real x_B0(min = 0, max= 1) = sorbent.x_B0 "[-], Initial conversion of calcium hydroxide " annotation(dialog(Enable=true));

  //sorbent properties
  final parameter Modelica.Units.SI.Concentration c_s_in = sorbent.c_s  "[mol/m3] Concerntration of sorbent";
  replaceable record Sorbent =
      ThermalPower.FlueGasTreatment.Records.Data.SCM_sorbent_CAOH2;
  Sorbent sorbent;

  final parameter Modelica.Units.SI.Density rho_s=sorbent.rho_s
    "Density of sorbent";
  final parameter Modelica.Units.SI.MolarMass M_s=sorbent.M_s
    "Molar weight of sorbent";

  final parameter Modelica.Units.SI.Radius r0=sorbent.r0 "Radius of particle";

  //1: external mass transfer
  parameter Modelica.Units.SI.Velocity k_g=0.175
    "Gas phase mass transfer coefficient";

  //2: reaction
  parameter Modelica.Units.SI.Velocity k_s0=1.55e-1 "frequency factor";
  parameter Real E_rec = 2.7e3 "[J/mol] activation Energy";
  Modelica.Units.SI.Velocity k_s "Reaction rate coeffcient";

  //3: diffusion
  parameter Real De0= 3.677e-9 "[m2/s] initial diffusion coefficient of gas through product layer";  //
  final parameter Real gamma0= to_J(0.0268) "[J/mol]";
  final parameter Real gamma1= to_J(0.275) "[J/mol]";
  Real De "[m2/s] Effective diffusion coefficient of SO2 through product layer";

  Real g_s "Sulfation rate";
  Modelica.Units.SI.Time tau_M "External mass transfer term";
  Modelica.Units.SI.Time tau_K "reaction term";
  Modelica.Units.SI.Time tau_D "Internal mass transfer term";
  Real x_B(start=0.5,min=0,max=1)  "Calcium hydroxide conversion ratio";

  Real eff "[0,1], SO2 removal efficiency";

  Modelica.Units.SI.Concentration c_r_out "Reactant concerntration of the sorbent at the outlet";

  Modelica.Units.SI.Density rho_s_out
    "Density of reacted sorbent (mixture of reactant and product)";
  Modelica.Units.SI.Density rho_r_out
    "Density of reactant kg(reactant)/m3(particle)";
  Modelica.Units.SI.MolarMass MM_s_out
    "Molar mass of reacted sorbent (mixture of reactant and product)";

  final parameter Real RS_g= 1  "Stoichiometry of SO2";
  final parameter Real RS_s= 1  "Stoichiometry of sorbent";

protected
  Real cscg "Ratio of c_s/c_g";

equation

  cscg = c_s_in;

  //1: SO2 mass transfer through gas phase
  tau_M = RS_g*r0/(3*RS_s*k_g)*cscg;

    //2: reaction
  k_s = k_s0*Modelica.Math.exp(-E_rec/R/T);
  tau_K = r0/(RS_s*k_s)*cscg;

    //3: internal/diffusion resistance through the product layer
  g_s = (gamma0 + gamma1/RH)/R/T;
  De = De0*Modelica.Math.exp(-g_s*x_B);
  tau_D = RS_g*r0^2/(6*RS_s*De)*cscg;

  eff = (x_B-x_B0);

  c_r_out = c_s_in*(1-x_B); // reactant concnerntration

  rho_r_out = rho_s*(1-x_B);
  rho_s_out =  c_s_in*(sorbent.M_s*(1-x_B)+sorbent.M_p*x_B);
  MM_s_out = sorbent.M_s*(1-x_B)+sorbent.M_p*x_B;

  annotation (experiment(
      StopTime=600,
      __Dymola_NumberOfIntervals=1000,
      Tolerance=1e-07,
      __Dymola_Algorithm="Dassl"), Documentation(info="<html>
      <p>Shrinking Core Model (SCM) describes the amount of gaseous pollutant that is being captured by a particle of given size while the unreacted core is shrinking. Knowing the residence time of the particle in the reactor, the reactant conversion rate and the pollutant concentration in the gas can easily be computed.
      </p>
      
      <p>3 resistances in series:
<p><img src=\"modelica://ThermalPower/Resources/Images/FlueGasTreatment/SCM_resistance_implicit.png\"/></p>
<p><img src=\"modelica://ThermalPower/Resources/Images/FlueGasTreatment/SCM_resistance.png\"/></p>

<p>At given bulk gas concentration:
<ul><li> 
Dynamics of the conversion rate (or radius)</li><li>
Explicit integration at constant gas concentration</li></ul>
</p>

</html>"));
end PartialShrinkingCoreModel;
