within FlueGasTreatment.SubComponents.BagFilter;
model BagFilter_NaHCO3_SO2_HCl "Single Sorbent for Duo-Pollutant"
  // NaHCO3 as sorbent for SO2 and HCl removal with recirculation

  import Modelon.ThermoFluid.Choices.InitOptions;
  import ThermalPower.FlueGasTreatment.Internal.Conversion.to_J;
  import Modelica.Constants.R;
  import Modelica.Constants.eps;
  import Modelica.Constants.pi;
  import Modelica.Math.Vectors.find;
  import ThermalPower.FlueGasTreatment.Internal.Conversion.from_Nm3;
  extends FlueGasTreatment.SubComponents.BagFilter.PartialBagVolume(
    redeclare package Gas =
        ThermalPower.FlueGasTreatment.Internal.Media.Gases.ReactionGas_withAsh,
    Xstart=fill(1/Gas.nS, Gas.nS),
    redeclare replaceable record Sorbent =
        ThermalPower.FlueGasTreatment.Records.Data.SCM_sorbent_NaHCO3_SO2_HCl,
    redeclare replaceable model Reaction =
        FlueGasTreatment.Reactions.SCM.SCM_BagFilter_NaHCO3_SO2_HCl,
    redeclare replaceable record BedGeo =
        ThermalPower.FlueGasTreatment.Records.Geometry.Geo_BagFilter,
    Tstart=453.15);

  extends ThermalPower.FlueGasTreatment.Internal.Icons.bag_filter_icon;
  //Recirculation

  parameter Boolean recirculation_on=true "If true, recirculation of sorbent is used";
  parameter Modelica.Units.SI.Pressure dp_lim=10500 "Limitation of dp";
  parameter Real recycle(
    min=0,
    max=1) = 0.8 "Recycle percentage";
  parameter Real cal_HCl=1.5 "Calibration factor for HCl resistance";
  parameter Real cal_SO2=3 "Calibration factor for SO2 resistance";
  parameter String[:] acid_gas={"SO2","HCl"} "Names of the adsorbate gases"
    annotation (Dialog(group="Sorbents"));
  parameter Modelica.Units.SI.Volume V0=loss_cake.H_cake_max*geo.Ac*x_left
    "Initial cake total volume";
  parameter Real x_left=0.3 "0-1, portion of the cake that stays on the filter";
  constant Modelica.Units.SI.Density ash_density=2500 "Ash density";
  constant Integer HCl=4;
  constant Integer Na2SO3=1;
  constant Integer NaCl=2;

  final parameter Integer N_ads(min=1) = size(acid_gas, 1)
    "Number of species adsorbed by the adsorbent";
  final parameter Integer[N_ads] adsorbate_index=Gas.traceIndexVector(acid_gas)
    "Index of of the adsorbate gases";

  Modelica.Units.SI.Height H_cake "Sorbent accumulation height";

  Modelica.Units.SI.MassFlowRate m_flow_s_out(start=0)
    "Mass flow rate of Reactant at the inlet";
  Modelica.Units.SI.Density rho_s_out=SCM.rho_s_out
    "Density of sorbent at the outlet";
  Modelica.Units.SI.Density rho_r_out=SCM.rho_r_out
    "Density of reactant at the outlet";

  Modelica.Units.SI.Volume V_avg(start=0.5*V0, fixed=true)
    "Total volume of sorbent particles in the cake";
  Modelica.Units.SI.Volume V_s(start=V0, fixed=true)
    "Total sorbent in the cake";
  Modelica.Units.SI.AmountOfSubstance n1(start=0, fixed=true)
    "Amount of sorbent that has reacted with gas 1, SO2";
  Modelica.Units.SI.AmountOfSubstance n2(start=0, fixed=true)
    "Amount og sorbent that has reacted with gas 2, HCl";

  //variables for sorbent balance
  Modelica.Units.SI.VolumeFlowRate dV_scm "Volume changes due to SCM";

  Real N_avg(nominal=1e15) "[-], Average number of sorbent particles in the cake";
  Modelica.Units.SI.Radius r_avg(nominal=1e-5)
    "Average radius of sorbent particles in the cake";
  Real x_B_out_sampled(start=0.5);
  Modelica.Units.SI.Mass m_Na2SO3;
  Modelica.Units.SI.Mass m_NaCl;
  Reaction SCM(
    redeclare package Gas = Gas,
    T=T,
    p=p,
    sorbent=sorbent,
    c_g_in_so2=max(1e-15, (0*c_g_in + 2*c_g_out)/2),
    c_g_in_hcl=max(1e-15, 0.5*(2*c_g_out2 + 0*c_g_in2)),
    rc_hcl=r_avg,
    rc_so2=r_avg,
    x_B0=x_B_in,
    x_B=x_B_out,
    y1=y1);

  //gas properties
  Modelica.Units.SI.Concentration c_g_out "Concerntration of gas";
  Modelica.Units.SI.MolarFlowRate[Gas.nC] Zc_flow "Component mass residual";
  Modelica.Units.SI.AmountOfSubstance[Gas.nC] Zc;
  Modelica.Units.SI.AmountOfSubstance[2] Z_product(start={15,80})
    "Amount of product formed";

  replaceable model Friction_filter =
      ThermalPower.FlueGasTreatment.Internal.Friction.Friction_BagFilter
    constrainedby
    ThermalPower.FlueGasTreatment.Internal.Friction.PartialFriction
    "Pressure drop model" annotation (choicesAllMatching=true, dialog(group="Pressure drop"));

  Friction_filter loss_cake(H_cake=H_cake, v_flow_gas=v_flow_gas);

  Modelica.Units.SI.VolumeFlowRate v_flow_gas;
  Modelica.Units.SI.Density rho_gas;

  Real[Gas.nC] prevent_negativeConcentration;
  Modelica.Units.SI.MolarFlowRate[Gas.nC] dZ_scm;

  Modelica.Units.SI.Concentration c_g_in2 "Concerntration of gas";
  Modelica.Units.SI.Concentration c_g_out2 "Concerntration of gas";

  Modelica.Units.SI.VolumeFlowRate dV_scm2 "Volume changes due to SCM2";
  Real y1 "Molar ratio of sorbent that has reacted with SO2";
  Real y1_sampled(start=0.5) "Molar ratio of sorbent that has reacted with SO2";
  Modelica.Units.SI.Mass m_cake;
  Modelica.Units.SI.Mass m_freshSorbent;
  Real y1_in;
  Modelica.Units.SI.Time period(start=600);

  Modelica.Units.SI.Time clean_time(start=1800);
  Modelica.Units.SI.Mass m_ash;
  Real x_Na2SO3;
  Real x_NaCl;
  Real x_ash;
  Real x_fresh;
  Modelica.Units.SI.MassFlowRate m_flow_cake "Mass flow of cake removed";
  input Modelica.Units.SI.MassFlowRate m_flow_ash "MAss flow of collected ash in filter";
  Modelica.Units.SI.MassFlowRate ash_cake_out;
  Real m_flow_cake_kgh "[kg/h]";

  ThermalPower.FlueGasTreatment.Records.Data.KPI_filter kpi(
    m_flow_s_in=m_flow_s_in,
    m_flow_cake=m_flow_cake,
    recycle=recycle,
    m_flow_s_out=m_flow_s_out,
    x_salt_SO3=x_Na2SO3,
    x_salt_Cl=x_NaCl,
    x_ash=x_ash,
    x_fresh=x_fresh,
    period=period)
    annotation (Placement(transformation(extent={{-90,-68},{-70,-48}})));
initial equation
  Zc = {Cstart[i]*0.01/Gas.MMC[i]*Ztot*MM for i in 1:Gas.nC};

equation
  y1_in = sorbent_in.y;
  rho_s_in = sorbent.rho_s*((1 - x_B_in) + x_B_in/sorbent.M_s*(sorbent.M_p*y1_in*sorbent.RC_Na2SO3 +
    sorbent.M_p_2*(1 - y1_in)*sorbent.RC_NaCl));
  effective_RS = 0;
  dp = loss_cake.dp;
  H_cake = (V_s + m_ash/ash_density)/geo.Ac;

  rho_gas = 0.72;
  // avoids equation systems

  v_flow_gas = feed.m_flow/rho_gas;

  dE = feed.m_flow*hi + drain.m_flow*ho + q.Q_flow + SCM.Q;

  for i in 1:Gas.nC loop
    prevent_negativeConcentration[i] = Modelon.Math.Smoothing.spliceFunction(
      1,
      0,
      V_avg,
      1e-6)*Modelon.Math.Smoothing.spliceFunction(
      1,
      0,
      Zc[i],
      1e-9);
    Zc_flow[i] = dMC[i]/Gas.MMC[i];
    C[i] = Zc[i]*Gas.MMC[i]/Ztot/MM;
    //
    if i == SO2 then
      dZ_scm[i] = dV_scm*sorbent.rho_s/sorbent.M_s*sorbent.RC_SO2;
      der(Zc[i]) = Zc_flow[i] + dZ_scm[i];
    elseif i == HCl then
      dZ_scm[i] = dV_scm2*sorbent.rho_s/sorbent.M_s*sorbent.RC_HCl;
      der(Zc[i]) = Zc_flow[i] + dZ_scm[i];
    else
      der(Zc[i]) = Zc_flow[i];
      dZ_scm[i] = 0;
    end if;
  end for;
  c_g_out = C[SO2]/Gas.MMC[SO2]/(1/MM*R*T/p);
  c_g_in2 = C_i[4]/Gas.MMC[4]/(1/MM*R*T/p);
  c_g_out2 = C[4]/Gas.MMC[4]/(1/MM*R*T/p);

  // balance of products
  der(Z_product[Na2SO3]) = -dV_scm*sorbent.rho_s/sorbent.M_s*sorbent.RC_Na2SO3;
  der(Z_product[NaCl]) = -dV_scm2*sorbent.rho_s/sorbent.M_s*sorbent.RC_NaCl;

  //volume balance of reactant
  der(V_avg) = vol_flow_r_in + dV_scm + dV_scm2;
  dV_scm = N_avg*4*pi*r_avg^2*SCM.scm_so2.drcdt*prevent_negativeConcentration[SO2]/cal_SO2;
  V_avg = 4/3*pi*N_avg*(Modelica.Constants.eps + r_avg)^3;
  // computation of r_avg
  //r_avg =(Modelica.Constants.eps+V_avg/(4/3*pi*N_avg))^(1/3);
  dV_scm2 = N_avg*4*pi*r_avg^2*SCM.scm_hcl.drcdt*prevent_negativeConcentration[HCl]/cal_HCl;

  m_Na2SO3 = Z_product[Na2SO3]*sorbent.M_p;
  m_NaCl = Z_product[NaCl]*sorbent.M_p_2;

  x_Na2SO3 = m_Na2SO3/m_cake;
  x_NaCl = m_NaCl/m_cake;
  x_ash = m_ash/m_cake;
  x_fresh = m_freshSorbent/m_cake;

  der(m_ash) = m_flow_ash + sorbent_in.m_ash;
  der(n1) = vol_flow_s_in*x_B_in/sorbent.M_s*y1_in - dV_scm*sorbent.rho_s/sorbent.M_s;
  der(n2) = vol_flow_s_in*x_B_in/sorbent.M_s*(1 - y1_in) - dV_scm2*sorbent.rho_s/sorbent.M_s;
  y1 = n1/(n1 + n2 + Modelica.Constants.eps);

  //balance in the sorbent particle mass
  der(V_s) = vol_flow_s_in;
  N_avg = max(1e-15, V_s)/sorbent.V_p;
  x_B_out = max(0, min(1, 1 - V_avg/V_s));

  m_cake = V_s*rho_s_out + m_ash;
  m_freshSorbent = V_avg*sorbent.rho_s;
  //cleaning process
  when loss_cake.dp > dp_lim then
    reinit(V_avg, 1e-15 + pre(V_avg)*x_left);
    //reinit(V_avg2,1e-15+pre(V_avg2)/3);
    reinit(Z_product[Na2SO3], pre(Z_product[Na2SO3])*x_left);
    reinit(Z_product[NaCl], pre(Z_product[NaCl])*x_left);
    reinit(V_s, 1e-15 + pre(V_s)*x_left);
    reinit(n1, 1e-15 + pre(n1)*x_left);
    reinit(n2, 1e-15 + pre(n2)*x_left);
    reinit(m_ash, 1e-15 + pre(m_ash)*x_left);
    clean_time = time;
    period = clean_time - pre(clean_time);

    x_B_out_sampled = pre(x_B_out);
    m_flow_s_out = recycle*(1 - x_left)*(pre(m_cake)-pre(m_ash))/max(1, pre(period));
    ash_cake_out = recycle*(1 - x_left)*pre(m_ash)/period;
    y1_sampled = if recirculation_on then pre(y1) else 0.5;
    m_flow_cake = (1 - x_left)*pre(m_cake)/max(1, pre(period));
    m_flow_cake_kgh = m_flow_cake*3600;
  end when;

  if recirculation_on then
    sorbent_out.m_sorbent = m_flow_s_out;
    sorbent_out.x_B = x_B_out_sampled;
    sorbent_out.m_ash = ash_cake_out;
    sorbent_out.y = y1_sampled;

  else
    sorbent_out.m_sorbent = m_flow_s_out/recycle;
    sorbent_out.x_B = 1;
    sorbent_out.m_ash = 0;
    sorbent_out.y = 0.5;

  end if;

  annotation (Placement(transformation(extent={{-46,22},{-26,42}})), experiment(
      StopTime=1000,
      Tolerance=1e-07,
      __Dymola_Algorithm="Dassl"),
    Documentation(info="<html>
    <p> The bag filter with shrinking core is to describe the acids’ neutralization (HCl, SO2) with the sorbent of NaHCO3. All particles of a given reactant including in the REFIOM stream will be assumed to have the same outer diameter. The different conversion rates of the reactant inside the particles are accounted for. 
</p>
<p>Three Limiting factors
<ul>
<li>Mass transfer in gas film around particle</li>
<li>Diffusion in porous product material</li>
<li>
Reaction kinetics</li>
</ul>
</p>

<h3>Modeling Approach</h3>
<p>Complexity
<ul>
<li>Single sorbent particle size</li>
<li>
Varying radius for unreacted core</li>
<li>
Unreacted core size: no local variation</li>
</ul>
</p>
<p>Shrinking core model with averaging
<ul>
<li>Mixing of fresh and REFIOM recirculation</li>
<li>
Sorbent particles in the cake</li>
</ul>
</p>
<p>
Three sub-models
<ul>
<li>Reactor: transport with reaction</li>
<li>
Bag filter: Cake + filter</li>
<li>
Refiom recirculation incl averaging</ul>
</p>
<p>
<p><img src=\"modelica://ThermalPower/Resources/Images/FlueGasTreatment/BagFilter.png\"/></p>

<h3>Model Interface</h3>

<p>Information needed at the port</p>
<ul>
<li>Mass flow of sorbent particles (total, weight based)</li>
<li>Conversion rate (to compute amount of unreacted core)</li>
<li>Molar fraction of the product that has reacted with SO2 (for density computation in the case of multi-species/single sorbent: NAHCO3)  
</li>
<li>Ash content in the cake</li>
</ul>
<h3>Cake Growth - multiple species</h3>
<p> Several sources of volume change
<ul>
<li>Reaction with substance 1
</li>
<li>Reaction with substance 2
</li>
<li>Each substance follows its own dynamics: (1) Assuming similar diffusion through gas & product; (2)Assuming Different reaction kinetics & bulk gas
</li>
</ul></p>
<p>Unchanged average radius computation
<ul>
<li>Based on total volume
</li>
</ul></p>
<p><img src=\"modelica://ThermalPower/Resources/Images/FlueGasTreatment/cake_growth_multi.png\"/></p>



<h3>Pressure drop model</h3>
<p>Filter loss model (independent of cake height)<ul><li>
Cake loss model (dependent on cake height)</li></ul></p>
<p>Event when pressure limit reached
<ul><li>
Mass fraction of cake left on the filter is a parameter
</li></ul>
</p>
<p>Part of the cake can be recirculated
<ul><li>
Mass fraction of the cake to be recirculate is a parameter
</li></ul>
</p>
</html>"));
end BagFilter_NaHCO3_SO2_HCl;
